﻿Imports System.Text
Imports Microsoft.VisualStudio.TestTools.UnitTesting
Imports System.Threading

Namespace Evolve.WindowsAzure.Tests
    <TestClass> _
    Public Class UniqueIdGeneratorTest
        <TestMethod> _
        Public Sub Test_Constructor_Uses_Store_Correctly_Under_Canonical_Load()
            Dim mock As New TestOptimisticSyncStore()
            mock.GetDataValue = "100"
            mock.TryWriteResult = True
            Assert.AreEqual(mock.SetDataValue, Nothing)
            Dim subject As New UniqueIdGenerator(mock, 2, 2)
            ' should retrieve 100, add the range to get a new upper limit of 102
            Assert.AreEqual(mock.SetDataValue, "102")
            Assert.AreEqual(101, subject.NextId())
            Assert.AreEqual(102, subject.NextId())
        End Sub

        <TestMethod, ExpectedException(GetType(Exception))> _
        Public Sub Test_Constructor_Blows_On_Corrupt_Store()
            Dim mock As New TestOptimisticSyncStore()
            mock.GetDataValue = "1oo"
            Dim subject As New UniqueIdGenerator(mock)
        End Sub

        <TestMethod, ExpectedException(GetType(Exception))> _
        Public Sub Test_Constructor_Blows_On_Null_Store()
            Dim mock As New TestOptimisticSyncStore()
            mock.GetDataValue = Nothing
            Dim subject As New UniqueIdGenerator(mock)
        End Sub

        <TestMethod> _
        Public Sub Test_NextId_Blows_On_Corrupt_Data()
            Dim mock As New TestOptimisticSyncStore()
            mock.GetDataValue = "100"
            mock.TryWriteResult = True
            'single digit range to ensure store is used for every id
            Dim subject As New UniqueIdGenerator(mock, 1, 2)
            mock.GetDataValue = "nonsense"
            subject.NextId()
            Try
                subject.NextId()
            Catch e1 As Exception
                Return
            End Try

            Assert.Fail("NextId should have thrown inside try block before this Fail Assertion")
        End Sub

        <TestMethod> _
        Public Sub Test_NextId_Deals_Numbers_Sequentially()
            Dim mock As New TestOptimisticSyncStore()
            mock.GetDataValue = "1"
            mock.TryWriteResult = True
            Dim subject As New UniqueIdGenerator(mock, 3, 0)
            mock.GetDataValue = "250"
            Assert.AreEqual(2, subject.NextId())
            Assert.AreEqual(3, subject.NextId())
            Assert.AreEqual(4, subject.NextId())
            Assert.AreEqual(251, subject.NextId())
            Assert.AreEqual(252, subject.NextId())
        End Sub

        <TestMethod> _
        Public Sub Test_Retries_Are_Exhausted()
            Dim mock As New TestOptimisticSyncStore()
            mock.GetDataValue = "0"
            mock.TryWriteResult = False
            Try
                Dim subject As New UniqueIdGenerator(mock, 3, 2)
            Catch exc As Exception
                Assert.AreEqual(3, mock.TryWriteCount)
                Assert.AreEqual("Failed to update the OptimisticSyncStore after 3 attempts", exc.Message)
                Return
            End Try
            Assert.Fail("NextId should have thrown and been caught in the try block")
        End Sub
    End Class

    Public Class TestOptimisticSyncStore
        Implements IOptimisticSyncStore
        Private _tryWriteCount As Integer = 0

        Private _GetDataValue As String
        Public Property GetDataValue() As String
            Get
                Return _GetDataValue
            End Get
            Set(ByVal value As String)
                _GetDataValue = value
            End Set
        End Property
        Private _SetDataValue As String
        Public Property SetDataValue() As String
            Get
                Return _SetDataValue
            End Get
            Private Set(ByVal value As String)
                _SetDataValue = value
            End Set
        End Property
        Private _TryWriteResult As Boolean
        Public Property TryWriteResult() As Boolean
            Get
                Return _TryWriteResult
            End Get
            Set(ByVal value As Boolean)
                _TryWriteResult = value
            End Set
        End Property
        Public ReadOnly Property TryWriteCount() As Integer
            Get
                Return _tryWriteCount
            End Get
        End Property

        Public Function GetData() As String Implements IOptimisticSyncStore.GetData
            Return GetDataValue
        End Function

        Public Function TryOptimisticWrite(ByVal data As String) As Boolean Implements IOptimisticSyncStore.TryOptimisticWrite
            Interlocked.Increment(_tryWriteCount)
            SetDataValue = data
            Return TryWriteResult
        End Function
    End Class

End Namespace
