﻿Imports System.Text
Imports Microsoft.WindowsAzure
Imports Microsoft.WindowsAzure.StorageClient

Namespace Evolve.WindowsAzure
    ''' <summary>
    ''' Uses blob storage to store the 'released' indicator. The mere existence of the file indicates
    ''' a release state in this implementation.
    ''' </summary>
    Public Class BlobGlobalFlag
        Implements IGlobalFlag
        Private ReadOnly _token As String = "TRUE!"
        Private ReadOnly _blobReference As CloudBlob
        Private ReadOnly _blobContainer As CloudBlobContainer

        Public Sub New(ByVal account As CloudStorageAccount, ByVal container As String, ByVal address As String)
            Dim blobClient = account.CreateCloudBlobClient()
            _blobContainer = blobClient.GetContainerReference(container.ToLower())
            _blobReference = _blobContainer.GetBlobReference(address)
        End Sub

        Public Sub SetFlag(ByVal status As Boolean) Implements IGlobalFlag.SetFlag
            If status Then
                _blobContainer.CreateIfNotExist()
                _blobReference.UploadText(_token)
            Else
                _blobReference.DeleteIfExists()
            End If
        End Sub

        Public Function GetFlag() As Boolean Implements IGlobalFlag.GetFlag
            Try
                _blobReference.DownloadText()
                Return True
            Catch exc As StorageClientException
                If exc.StatusCode = System.Net.HttpStatusCode.NotFound Then
                    Return False
                End If
                Throw
            End Try
        End Function
    End Class
End Namespace
