﻿Imports System.Text
Imports Microsoft.WindowsAzure
Imports Microsoft.WindowsAzure.StorageClient
Imports System.Net

Namespace Evolve.WindowsAzure
    ''' <summary>
    ''' Stores a single string value in Blob storage and provides an easy way to update 
    ''' the value using Optimistic Concurrency.
    ''' </summary>
    Public Class BlobOptimisticSyncStore
        Implements IOptimisticSyncStore
        Private ReadOnly _blobReference As CloudBlob

        Public Sub New(ByVal account As CloudStorageAccount, ByVal container As String, ByVal address As String)
            Dim blobClient = account.CreateCloudBlobClient()
            Dim blobContainer = blobClient.GetContainerReference(container.ToLower())
            _blobReference = blobContainer.GetBlobReference(address)
        End Sub

        Public Function GetData() As String Implements IOptimisticSyncStore.GetData
            Dim data As String = _blobReference.DownloadText()
            Return data
        End Function

        Public Function TryOptimisticWrite(ByVal data As String) As Boolean Implements IOptimisticSyncStore.TryOptimisticWrite
            Try
                _blobReference.UploadText(data, Encoding.Default, New BlobRequestOptions With {.AccessCondition = AccessCondition.IfMatch(_blobReference.Properties.ETag)})
            Catch exc As StorageClientException
                If exc.StatusCode = HttpStatusCode.PreconditionFailed Then
                    Return False
                Else
                    Throw
                End If
            End Try
            Return True
        End Function
    End Class
End Namespace
