﻿Imports System.Text
Imports Microsoft.WindowsAzure.StorageClient
Imports Evolve.WindowsAzure
Imports Microsoft.WindowsAzure
Imports Microsoft.ServiceBus
Imports System.ServiceModel
Imports System.Net
Imports System.Configuration

Namespace TestClient
    ''' <summary>
    ''' Interaction logic for MainWindow.xaml
    ''' </summary>
    Partial Public Class MainWindow
        Inherits Window
        Private Shared ReadOnly _serviceNamespace As String = ConfigurationManager.AppSettings("ServiceBusNamespace")
        Private Shared ReadOnly _issuerName As String = ConfigurationManager.AppSettings("ServiceBusIssuerName")
        Private Shared ReadOnly _issuerSecret As String = ConfigurationManager.AppSettings("ServiceBusIssuerSecret")

        Private Const _uniqueIdContainer As String = "uniqueids"

        Private ReadOnly _globalFlag As IGlobalFlag = New BlobGlobalFlag(CloudStorageAccount.DevelopmentStorageAccount, "globalflags", "start-order-processing.dat")
        Private ReadOnly _store As IOptimisticSyncStore = New BlobOptimisticSyncStore(CloudStorageAccount.DevelopmentStorageAccount, _uniqueIdContainer, "ordernumber.dat")

        Public Sub New()
            InitializeComponent()
            UpdateFlagStatus()
            UpdateCurrentValue()
        End Sub

        Private Sub ListeningReleaseButton_Click(ByVal sender As Object, ByVal e As RoutedEventArgs)
            'Create the credentials object for the endpoint.
            Dim sharedSecretServiceBusCredential As New TransportClientEndpointBehavior()
            sharedSecretServiceBusCredential.CredentialType = TransportClientCredentialType.SharedSecret
            sharedSecretServiceBusCredential.Credentials.SharedSecret.IssuerName = _issuerName
            sharedSecretServiceBusCredential.Credentials.SharedSecret.IssuerSecret = _issuerSecret

            Dim address As Uri = ServiceBusEnvironment.CreateServiceUri("sb", _serviceNamespace, "UnleashEventService")

            Dim channelFactory = New ChannelFactory(Of IUnleashContract)("UnleashEndpoint", New EndpointAddress(address))
            channelFactory.Endpoint.Behaviors.Add(sharedSecretServiceBusCredential)
            Dim channel = channelFactory.CreateChannel()

            ' Release the hounds
            channel.Unleash()
        End Sub

        Private Sub PollingReleaseButton_Click(ByVal sender As Object, ByVal e As RoutedEventArgs)
            _globalFlag.SetFlag(True)
            UpdateFlagStatus()
        End Sub

        Private Sub PollingResetButton_Click(ByVal sender As Object, ByVal e As RoutedEventArgs)
            _globalFlag.SetFlag(False)
            UpdateFlagStatus()
        End Sub

        Private Sub ResetOrderIdFileButton_Click(ByVal sender As Object, ByVal e As RoutedEventArgs)
            Try
                Dim data As String = _store.GetData()
                ' if it doesn't blow here, then the file already exists, check user wants to progress
                Dim proceed = MessageBox.Show(String.Format("CAUTION: The file already contains the value '{0}'. If you overwrite this you could cause corruption by reusing existing order IDs. Proceed anyway?", data), "Warning", MessageBoxButton.YesNo, MessageBoxImage.Warning)

                If proceed <> MessageBoxResult.Yes Then
                    ' jump out
                    Return
                End If
            Catch exc As StorageClientException
                If exc.StatusCode = HttpStatusCode.NotFound Then
                    ' try to create the container in case it doesn't exist.
                    Dim blobClient = CloudStorageAccount.DevelopmentStorageAccount.CreateCloudBlobClient()
                    Dim blobContainer = blobClient.GetContainerReference(_uniqueIdContainer)
                    blobContainer.CreateIfNotExist()
                    ' everything is ok... progress
                Else
                    Throw
                End If
            End Try
            _store.TryOptimisticWrite("0")
            UpdateCurrentValue()
        End Sub

        Private Sub RefreshFlagStatus_Click(ByVal sender As Object, ByVal e As RoutedEventArgs)
            UpdateFlagStatus()
        End Sub

        Private Sub UpdateFlagStatus()
            FlagStatusText.Text = _globalFlag.GetFlag().ToString()
        End Sub

        Private Sub RefreshCurrentValue_Click(ByVal sender As Object, ByVal e As RoutedEventArgs)
            UpdateCurrentValue()
        End Sub

        Private Sub UpdateCurrentValue()
            Try
                CurrentValueText.Text = _store.GetData()
                CurrentValueText.Foreground = Brushes.Black
            Catch exc As StorageClientException
                If exc.StatusCode = HttpStatusCode.NotFound Then
                    CurrentValueText.Text = "No file"
                    CurrentValueText.Foreground = Brushes.Red
                Else
                    Throw
                End If
            End Try
        End Sub


    End Class
End Namespace
