﻿Imports System.Net
Imports System.Text
Imports System.Threading
Imports Microsoft.WindowsAzure.Diagnostics
Imports Microsoft.WindowsAzure.ServiceRuntime
Imports Evolve.WindowsAzure
Imports Microsoft.WindowsAzure
Imports System.Data.SqlClient
Imports System.Configuration
Imports System.Threading.Tasks

Namespace WorkerRole1
'    
'    
'     SQL Script to create table
'     ==========================
'     *
'        USE [SynchronizingAzure]
'        GO
'
'        IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[UniqueIdGeneratorTest]') AND type in (N'U'))
'        DROP TABLE [dbo].[UniqueIdGeneratorTest]
'        GO
'
'        USE [SynchronizingAzure]
'        GO
'
'        SET ANSI_NULLS ON
'        GO
'
'        SET QUOTED_IDENTIFIER ON
'        GO
'
'        CREATE TABLE [dbo].[UniqueIdGeneratorTest](
'            [UniqueId] [int] NOT NULL,
'            [DeploymentId] [nvarchar](250) NOT NULL,
'            [ThreadId] [int] NOT NULL
'        ) ON [PRIMARY]
'
'        GO
'    
'     
    Public Class WorkerRole
        Inherits RoleEntryPoint
        Private _generator As UniqueIdGenerator

        Public Overrides Sub Run()
            Dim serviceBusNamespace As String = ConfigurationManager.AppSettings("ServiceBusNamespace")
            Dim issuerName As String = ConfigurationManager.AppSettings("ServiceBusIssuerName")
            Dim issuerSecret As String = ConfigurationManager.AppSettings("ServiceBusIssuerSecret")

            Dim connectedRelease As New ListeningRelease(serviceBusNamespace, "UnleashEventService", issuerName, issuerSecret)
            Trace.WriteLine("LISTENING: Waiting for ServiceBus Signal to release")
            connectedRelease.Wait()

            Dim globalFlag As New BlobGlobalFlag(CloudStorageAccount.DevelopmentStorageAccount, "globalflags", "start-order-processing.dat")
            Dim pollingRelease As New PollingRelease(globalFlag, 2500)
            Trace.WriteLine("POLLING: Waiting for Global Flag to be set to be released")
            pollingRelease.Wait()

            Trace.WriteLine("Creating OptimisticSyncStore", "Information")
            Dim storage As IOptimisticSyncStore = New BlobOptimisticSyncStore(CloudStorageAccount.DevelopmentStorageAccount, "uniqueids", "ordernumber.dat")
            Trace.WriteLine("Creating UniqueIdGenerator", "Information")
            _generator = New UniqueIdGenerator(storage, 100, 10)

            Do
                Thread.Sleep(50)

                ' Using the threadpool ensures that the generator is also good for thread safe access within
                ' a given app domain - using Tasks in a relatively inefficent way for brevity and simplicity
                ' (and to avoid exhausting the ThreadPool as it is required for requests to Blob Storage etc)
                Dim t1 As Task = Task.Factory.StartNew(AddressOf GenerateAndStoreOrderId)
                Dim t2 As Task = Task.Factory.StartNew(AddressOf GenerateAndStoreOrderId)
                Dim t3 As Task = Task.Factory.StartNew(AddressOf GenerateAndStoreOrderId)

                ' Wait until all three have finished before kicking off another bunch (to avoid exhausting the ThreadPool).
                Task.WaitAll(t1, t2, t3)
            Loop
        End Sub

        Private Sub GenerateAndStoreOrderId()
            Thread.Sleep(10)
            ' Generate a new order id and save it to the database.
            Dim orderId As Int64 = _generator.NextId()
            Trace.WriteLine(String.Format("New order Id allocated: {0}", orderId), "Information")

            Using conn As New SqlConnection(ConfigurationManager.ConnectionStrings("testDB").ConnectionString)
            Using cmd As SqlCommand = conn.CreateCommand()
                cmd.CommandText = "INSERT INTO UniqueIdGeneratorTest (UniqueId, DeploymentId, ThreadId) VALUES (@UniqueId, @DeploymentId, @ThreadId)"
                cmd.Parameters.Add(New SqlParameter("@UniqueId", orderId))
                cmd.Parameters.Add(New SqlParameter("@DeploymentId", RoleEnvironment.DeploymentId))
                cmd.Parameters.Add(New SqlParameter("@ThreadId", Thread.CurrentThread.ManagedThreadId))
                conn.Open()
                cmd.ExecuteNonQuery()
            End Using
            End Using
        End Sub

        Public Overrides Function OnStart() As Boolean
            ' Set the maximum number of concurrent connections 
            ServicePointManager.DefaultConnectionLimit = 12

            DiagnosticMonitor.Start("DiagnosticsConnectionString")

            ' For information on handling configuration changes
            ' see the MSDN topic at http://go.microsoft.com/fwlink/?LinkId=166357.
            AddHandler RoleEnvironment.Changing, AddressOf RoleEnvironmentChanging

            Return MyBase.OnStart()
        End Function

        Private Sub RoleEnvironmentChanging(ByVal sender As Object, ByVal e As RoleEnvironmentChangingEventArgs)
            ' If a configuration setting is changing

            If e.Changes.Any(Function(change) TypeOf change Is RoleEnvironmentConfigurationSettingChange) Then
                ' Set e.Cancel to true to restart this role instance
                e.Cancel = True
            End If

        End Sub

    End Class
End Namespace
